<?php if (!defined('PmWiki')) exit();
# vim: set ts=4 sw=4 et:
##
##        File: WikiShCL.php
##     Version: 2008-07-13
##      SVN ID: $Id: WikiShCL.php 386 2009-11-14 14:41:24Z root $
##      Status: alpha
##      Author: Peter Bowers
## Create Date: July 13, 2008
##   Copyright: 2008, Peter Bowers
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License, Version 2, as
## published by the Free Software Foundation.
## http://www.gnu.org/copyleft/gpl.html
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

# This recipe is a sub-recipe from WikiSh.  It provides a command-line form
# through the markup (:wikish_controlpanel:) or (:wikish_command_line:) 
# (synonyms)

$RecipeInfo['WikiShCL']['Version'] = '2008-07-13';

# (:wikish_command_line:) provides a form-based command-line interface for 
# wikish.  Facilities such as history, favorites, debugging, etc. are provided.
Markup('wshControlPanel', '<{$var}',
  '/\\(:wikish_(?:controlpanel|command_?line)\\s*(\\w*):\\)/e',
  "wshControlPanel(\$pagename, strtoupper('$1'))");
function wshControlPanel($pagename, $method)
{
    global $InputValues, $FmtPV, $WikiShVars;
    global $HTMLStylesFmt;
    $func = 'ControlPanel()';
    $histwidth=95;
    if ($method == "GET")
        $method_array = $_GET;
    else {
        $method = 'POST';
        $method_array = $_POST;
    }
    foreach ($method_array as $k=>$v) {
       $foo = htmlspecialchars(stripmagic($v),ENT_NOQUOTES);
       $InputValues[$k] = $foo;
       $foo = preg_replace('/\\\\/', '\\\\\\\\\\\\', $foo);
       $foo = preg_replace("/(?<!\\\)'/", "\'", $foo);
       $foo = preg_replace("/\&quot;/", '"', $foo);
       # This creates a Page Variable for use in the rest of the page
       wdbg(0,"$func: Setting $k => $foo");
       $FmtPV['$'.$k] = "'$foo'";
       $WikiShVars[$k] = "$foo";
	   if ($k == 'DEFAULT_DEBUG') {
		   #echo "Setting debug to $foo<br>\n";
		   $WikiShVars['DEBUGLEVEL'] = "$foo";
	   }
    }
    if (!$WikiShVars['panel_dbg']) $WikiShVars['DEBUGLEVEL'] = 5;
    #$InputValues['cmd'] = str_replace('&crarr;', CHR(13), wshUnHTMLSpecialChars($InputValues['cmd']), true);
    $InputValues['cmd'] = wshUnHTMLSpecialChars($InputValues['cmd']);
    #Clear command if clear is set
    if (isset($InputValues['clear'])) {
        wdbg(1,"$func: clearing commmand");
        $InputValues['cmd'] = '';
    }
    if (!$InputValues['NOHIST']) {
        #Read in history and favorites (2-dim array)
        $favsarray = wshReadHist($pagename, $opt, $WikiShVars['FAVSFILE']);
        $histarray = wshReadHist($pagename, $opt, $WikiShVars['HISTFILE']);
        # Now re-set the "cmd" if do_favs or load_favs was set
        if ($InputValues['do_favs'] || $InputValues['load_favs']) {
            $favsidx = $InputValues['favs'];
            #wdbg(0,"$func: do_favs index = $favsidx.  array follows");
            #wdbg(0,$favsarray);
            foreach ($favsarray as $flds) {
                if ($flds[0] == $favsidx) {
                    $InputValues['cmd'] = str_replace('&crarr;',"\n", $flds[1]);
                    break;
                }
            }
            if ($InputValues['do_favs']) $InputValues['do_cmd'] = true;
        }
        # Now re-set the "cmd" if do_hist or load_hist was set
        if ($InputValues['do_hist'] || $InputValues['load_hist']) {
            $histidx = $InputValues['hist'];
            foreach ($histarray as $flds) {
                if ($flds[0] == $histidx) {
                    $InputValues['cmd'] = str_replace('&crarr;',"\n",$flds[1]);
                    break;
                }
            }
            if ($InputValues['do_hist']) $InputValues['do_cmd'] = true;
        }
        # Copy to favorites if cp2favs was set
        if ($InputValues['cp2favs']) {
            $histidx = $InputValues['hist'];
            foreach ($histarray as $flds) {
                if ($flds[0] == $histidx) {
                    $fav2cp = $flds[1];
                    break;
                }
            }
            wdbg(1,"$func: Writing fav >>$fav2cp<< to history ");
            wshWriteHist($pagename, $opt, $WikiShVars['FAVSFILE'], wshUnHTMLSpecialChars($fav2cp));
            $need_re_read = true;
        }
        # Delete from favorites if del_favs was set
        if ($InputValues['del_favs']) {
            wdbg(1,"$func: del_favs index = $InputValues[favs]. array follows");
            wdbg(1,$favsarray);
            $fav2del = $InputValues['favs']; // need index, have index already
            wdbg(1,"$func: un-favoriting fav=>>$fav2del<<");
            wshDeleteHist($pagename, $opt, $WikiShVars['FAVSFILE'], $fav2del);
            $need_re_read = true;
        }
        # Write about-to-be-executed command to history
        if (isset($InputValues['do_cmd'])) {
            $cmd = str_replace(array(PHP_EOL, "\n"), '&crarr;', $InputValues['cmd']);
            wdbg(1,"$func: Writing to history $cmd");
            wshWriteHist($pagename, array('loc'=>$InputValues['HIST_LOC'], 'nodups'=>$InputValues['HIST_NODUPS'], 'max'=>$WikiShVars['HISTSIZE']), $WikiShVars['HISTFILE'], $cmd);
            $need_re_read = true;
        }
        if ($need_re_read) {
            $favsarray = wshReadHist($pagename, $opt, $WikiShVars['FAVSFILE']);
			wdbg(1,"$func: favsarray:"); wdbg(1,$favsarray);
            $histarray = wshReadHist($pagename, $opt, $WikiShVars['HISTFILE']);
			wdbg(1,"$func: histarray:"); wdbg(1,$histarray);
        }
        $histcnt = sizeof($histarray);
    }

    $HTMLStylesFmt['wikish-cp'] = "
                .wikishpanel { width: 100%; border:2px solid #999; }
        ";
    ## Now start building the form...
    $rtn =  "(:input form method=$method:)";
    $rtn .= '(:input hidden n {*$FullName}:)' . "\n";
    $rtn .= "(:input textarea name=cmd rows=8 cols=80 class='wikishpanel':)\\\\\n\n";
    $rtn .= "(:input submit do_cmd \"Execute Command\":) ";
    $rtn .= "(:input submit clear \"Clear\":)\\\\\n\\\\\n\n\n\n";
    $command = '{(wikish ' . wshParseCode($InputValues['cmd'], true) . ')}';
    $rtn .= "(:linebreaks:)";
    $rtn .= "\n(:table border=1 width=100% cellpadding=2px:)";
    $rtn .= "\n(:cell width=85 style='text-align:right; font-weight:bold;':)COMMAND";
    $rtn .= "\n(:cell:)" . Keep(htmlspecialchars($command));
    $rtn .= "\n(:cellnr style='text-align:right; font-weight:bold;':)OUTPUT";
    wdbg(1,"$func: Checking do_cmd");
    if (isset($InputValues['do_cmd'])) {
        $rtn .= "\n(:cell:)" . $command;
    } else {
        $rtn .= "\n(:cell:)(command not yet executed... Click on \"Execute Command\")";
    }
    $rtn .= "\n(:tableend:)\n";
    $rtn .= "\\\\\n\n";
    if (!$InputValues['NOHIST']) {
        # Begin a table which will hold history on the left and favorites on 
        # the right with appropriate buttons underneath each
        $rtn .= "\n(:table border=0 width=100%:)\n";
        $rtn .= "\n(:cell style='text-align:left; font-weight:bold;':)[[$WikiShVars[HISTFILE]?action=edit|History]]";
        if ($histcnt>$WikiShVars['HISTSIZE']) $rtn.=" ''(NOTE: History file Needs purging: $histcnt)''";
        $rtn .= "\n(:cell style='text-align:left; font-weight:bold;':)[[$WikiShVars[FAVSFILE]?action=edit|Favorites]]\n";
        #
        # Create "history" SELECT box
        #
        $i = 0;
        $rtn .= "\n(:cellnr:) ";
        if ($histarray) {
            if (sizeof($histarray) > $WikiShVars['HISTSIZE']) 
                $histarray = array_slice($histarray, 0, $InputValues['HISTSIZE']);
            foreach ($histarray as $hist) {
                if ($i != 0) $rtn .= "\n";
                $tmp = str_replace(array(":"), array("&#58;"), $hist[1]); // :)
                $tmp = htmlspecialchars($tmp);
                $tmp = str_replace(array("&amp;"),array('&'),$tmp); // &crarr;
                $rtn .= "(:input select name=hist value=$hist[0] label=\"[=" . ((strlen($tmp) > $histwidth) ? substr($tmp, 0, $histwidth-3).'...=]' : wshStrPad($tmp."=]", $histwidth, '&nbsp;')) . "\" " . (($i == 0) ? "size=5 style=\"width:95%\"" : "") . ":)";
                wdbg(1,"$func: setting hist select " . wshDbgOd(substr($hist[1], 3)));
                $i++;
            }
        }
        else
            $rtn .= "No History.  Use \"Execute Command\" button to create history.";
        #
        # Create "favorites" SELECT box
        #
        $i = 0;
        $rtn .= "\n(:cell:)";
        if ($favsarray)
            foreach ($favsarray as $favs) {
                if ($i != 0) $rtn .= "\n";
                $tmp = str_replace(array(":"), array("&#58;"), $favs[1]); // :)
                $tmp = htmlspecialchars($tmp);
                $tmp = str_replace(array("&amp;"),array('&'),$tmp); // &crarr;
                $rtn .= "(:input select name=favs value=$favs[0] \"[=" . ((strlen($tmp) > $histwidth) ? substr($tmp, 0, $histwidth-3).'...=]' : wshStrPad($tmp."=]", $histwidth, '&nbsp;')) . "\" " . (($i == 0) ? "size=5 style=\"width:95%\"" : "") . ":)";
                wdbg(1,"$func: setting fav select >>".wshDbgOd($favs[1])."<<");
                $i++;
            }
        else
            $rtn .= "No Favorites.  Use \"Copy to Favorites\" button under history.";
        $rtn .= "\n\n";
        #
        # Create history/favorites buttons
        #
        $rtn .= "\n(:cellnr:)(:input submit do_hist \"Execute History Command\":) (:input submit load_hist \"Load History Command\":) (:input submit cp2favs \"Copy To Favorites\":)";
        $rtn .= "\n(:cell:)(:input submit do_favs \"Execute Favorites Command\":) (:input submit load_favs \"Load Favorites Command\":) (:input submit del_favs \"Delete From Favorites\":)";
        $rtn .= "\n(:tableend:)\n";
    }
    # Set some defaults
    $rtn .= "\n(:input default DEFAULT_DEBUG $WikiShVars[DEFAULT_DEBUG]:)";
    $rtn .= "\n(:input default HISTSIZE $WikiShVars[HISTSIZE]:)";
    $rtn .= "\n(:input default NOHIST $WikiShVars[NOHIST]:)";
    $rtn .= "\n(:input default HIST_NODUPS nodups:)";
    $rtn .= "\n(:input default HIST_LOC 'prepend':)";
    $rtn .= "\n\n||";
    $rtn .= "\n||! Control Panel Interface Settings ||||";
    $rtn .= "\n|| Debug Level:||(:input text name=DEFAULT_DEBUG:) ''(1-5, 1=very detailed, 5=nothing)''||";
    $rtn .= "\n|| Control Panel Debug:||(:input checkbox name=panel_dbg value=checked:)||";
    $rtn .= "\n|| History Count:||(:input text name=HISTSIZE:)||\n";
    $rtn .= "\n|| History Location:||(:input select name=HIST_LOC value=prepend label=prepend:) (:input select name=HIST_LOC value=append label=append:)||\n";
    $rtn .= "\n|| Prevent Sequential Duplicates in History:||(:input checkbox name=HIST_NODUPS value=nodups:)||\n";
    $rtn .= "\n|| Suppress History & Favorites:||(:input checkbox name=NOHIST value=checked:) ''(Improves performance when HISTFILE is large)''||\n";
    $rtn .= "\n(:input end:)";
    $rtn .= "\n\\\\\n\\\\\n====\nDEBUG\n(:messages:)\n";
    wdbg(0,"$func: returning >>" . wshDbgOd($rtn) . "<<");
    wdbg(4,"$func: Exiting");
    $WikiShVars['DEBUGLEVEL'] = $WikiShVars['DEFAULT_DEBUG'];
    return($rtn);
}

#Simple delete from history/favorites file:
# $pagename    - pagename for reference
# $logfile     - history/favorites pagename
# $opt         - option array 
#                 - no options defined at this point
# $index       - index (timestamp) which identifies line to be deleted
#
# Note that permissions are *assumed* - check 'em before you call me!
function wshDeleteHist($pagename, $opt, $logfile, $index, $ifs=':')
{
	global $WikiShVars;
	$func = 'wshDeleteHist()';
	wdbg(4,"$func: Entering: $index");
	$logfile = wshMakePageName($pagename, $opt, $logfile);
    $page = ReadPage($logfile, READPAGE_CURRENT);
    $l = explode("\n", $page['text']); // timestamp:cmd
	$idxlen = strlen($index) + 1;
	wdbg(1,"$func: Lines READ follows:"); wdbg(1,$l);
	if ($WikiShVars['HIST_PREFIX'])
		$i = sizeof(explode("\n", $WikiShVars['HIST_PREFIX']));
	else
		$i = 0;
	while ($i < sizeof($l)) {
		if (strncmp($index.$ifs, $l[$i], $idxlen) == 0) {
			unset($l[$i]);
			break;
		}
		$i++;
	}
	wdbg(1,"$func: After deleting line ($i)"); wdbg(1,$l);
    $page['text'] = implode("\n", $l);
    WritePage($logfile, $page);
	return(true);
}

#Simple write to history/favorites file:
# $pagename    - pagename for reference
# $logfile     - history/favorites pagename
# $opt         - option array 
#                 - $opt['max'] can contain max # of lines (additional lines 
#                   will be deleted from the beginning
#                 - $opt['nodups'] disallows consecutive duplicates to be added
# $text        - text which will be written
#
# Note that permissions are *assumed* - check 'em before you call me!
function wshWriteHist($pagename, $opt, $logfile, $text, $ifs=':')
{
	global $WikiShVars;
	$func = "wshWriteHist($pagename)";
	wdbg(4,"$func: Entering: $text");
	$logfile = wshMakePageName($pagename, $opt, $logfile);
	wdbg(1,"$func: after MkPgNm: logfile=$logfile");
    $page = ReadPage($logfile, READPAGE_CURRENT);
    $l = explode("\n", $page['text']); // timestamp:cmd
	wdbg(1,"$func: Lines READ follows:"); wdbg(1,$l);
	if ($WikiShVars['HIST_PREFIX']) {
		$prefix_lines = sizeof(explode("\n", $WikiShVars['HIST_PREFIX']));
		$l = array_slice($l, $prefix_lines);
	}
	wdbg(1,"$func: After slicing prefix:"); wdbg(1,$l);
    SDV($opt['loc'], 'append');
	# Check for duplicates
	if (@$opt['nodups']) {
		if ($opt['loc'] == 'prepend')
			$x = $l[0];
		else
			$x = end($l);
		$lastcmd = end(explode(":", $x, 2));
		if ($lastcmd == $text) {
			wdbg(3,"$func: Duplicate - returning without doing anything.");
			return(2); // don't continue if there's no change to the logfile
		} // else echo "differ: lastcmd=>>$lastcmd<<, text=>>$text<<<br>\n";
	}
	# Prepend or append the newest history/favorite line onto the array
	$myline = time() . $ifs . $text;
	if ($opt['loc'] == 'prepend')
		array_unshift($l, $myline);
	else
		$l[] = $myline;
	wdbg(1,"$func: Line to be added is >>$myline<<");
	wdbg(1,"$func: After adding line ($opt[loc]):"); wdbg(1,$l);
	# Check for exceeded maximums and slice part of the array if needed
    if (@$opt['max'] && sizeof($l) > $opt['max']) {
        if ($opt['loc'] == 'prepend') {
            $start = 0; $len = $opt['max'];
        } else {
            $start = sizeof($l)-$opt['max']+1; $len = $opt['max'];
        }
        $l = array_slice($l, $start, $len);
		wdbg(1,"$func: After slicing exceeded lines ($start,$end):");wdbg(1,$l);
    }
    $page['text'] = $WikiShVars['HIST_PREFIX'] . "\n" . implode("\n", $l);
    WritePage($logfile, $page);
	return(true);
}

# Read from history file
function wshReadHist($pagename, $opt, $logfile, $ifs=':', $fldcnt=2)
{
	global $WikiShVars;
	$func = "wshReadHist($pagename)";
	wdbg(4,"$func: Entering: $text");
	$logfile = wshMakePageName($pagename, $opt, $logfile);
	wdbg(1,"$func: after MkPgNm: logfile=$logfile");
    $page = ReadPage($logfile, READPAGE_CURRENT);
	if (!$page) return(array());
    $l = explode("\n", $page['text']); // timestamp:cmd
	wdbg(1,"$func: Lines READ follows:"); wdbg(1,$l);
	if ($WikiShVars['HIST_PREFIX'])
		$i = sizeof(explode("\n", $WikiShVars['HIST_PREFIX']));
	else
		$i = 0;
	$rtn = array();
	while ($i < sizeof($l))
		$rtn[] = explode($ifs, $l[$i++], $fldcnt);
    return($rtn);
}

# Pad a string with &nbsp to the desired length
function wshStrPad($str, $len)
{
    if (strlen($str) >= $len)
        return ($str);
    for ($i = strlen($str); $i < $len; $i++)
        $str .= '&nbsp;';
    return ($str);
}

